@@ -1,6 +1,6 @@
 /*******************************************************************************
  *
- * Copyright (c) 2012-2019 Robert Krause (ruport@f00l.de)
+ * Copyright (c) 2012-2021 Robert Krause (ruport@f00l.de)
  *
  * This file is part of Pcapfix.
  *
@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU General Public License along with
  * Pcapfix. If not, see http://www.gnu.org/licenses/.
  *
- * Last Modified: 22.01.2019
+ * Last Modified: 03.07.2021
  *
  *******************************************************************************
  *
@@ -45,7 +45,7 @@
 #include "pcap_kuznet.h"
 #include "pcapng.h"
 
-#define VERSION "1.1.4"			    /* pcapfix version */
+#define VERSION "1.1.7"		    /* pcapfix version */
 
 #define BTSNOOP_MAGIC 0x6E737462    /* btsnoop file magic (first 4 bytes) */
 #define SNOOP_MAGIC 0x6f6f6e73	    /* snoop file magic (first 4 bytes) */
@@ -132,14 +132,14 @@ unsigned int conint(unsigned int var) {
  * filesize:  the size of the input pcap file in bytes
  *
  */
-void print_progress(uint64_t pos, uint64_t filesize) {
+void print_progress(off_t pos, off_t filesize) {
   float percentage;	/* pencentage variable */
 
   /* calculate the current percentage of file analyzing progress */
-  percentage = (float)pos/(float)filesize;
+  percentage = (float)(100*(double)pos/(double)filesize);
 
   /* print the first part of the line including percentage output */
-  printf("[*] Progress: %6.02f %%\n", percentage*100);
+  printf("[*] Progress: %6.02f %%\n", percentage);
 }
 
 /*
@@ -176,8 +176,8 @@ int main(int argc, char *argv[]) {
   char *filename;               /* filename of input file */
   char *filebname;              /* filebasename of input file (without path) */
   char *filename_fix = NULL;    /* filename of output file */
-  uint64_t bytes;		        /* read/written blocks counter */
-  uint64_t filesize;	        /* file size of input pcap file in bytes */
+  off_t bytes;		        /* read/written blocks counter */
+  off_t filesize;	        /* file size of input pcap file in bytes */
 
   /* init getopt_long options struct */
   struct option long_options[] = {
@@ -192,7 +192,7 @@ int main(int argc, char *argv[]) {
   };
 
   /* print out pcapfix header information */
-  printf("pcapfix %s (c) 2012-2019 Robert Krause\n\n", VERSION);
+  printf("pcapfix %s (c) 2012-2021 Robert Krause\n\n", VERSION);
 
   /* scan for options and arguments */
   while ((c = getopt_long(argc, argv, ":t:ko:v::d::s::n::", long_options, &option_index)) != -1) {
@@ -273,7 +273,7 @@ int main(int argc, char *argv[]) {
   // is output == inputfile ?? if yes, then open for read / append
   // if no, then open for writing only
   if (strcmp(filename, filename_fix) == 0) pcap_fix = fopen(filename_fix, "rb+");
-  else pcap_fix = fopen(filename_fix, "w+");
+  else pcap_fix = fopen(filename_fix, "wb+");
 
   if (!pcap_fix) {
     perror("[-] Cannot open output file for writing");
@@ -286,14 +286,17 @@ int main(int argc, char *argv[]) {
   /* get file size */
   fseeko(pcap, 0, SEEK_END);
   filesize = ftello(pcap);
-  printf("[*] File size: %" PRIu64 " bytes.\n", filesize);
+  printf("[*] File size: %" FMT_OFF_T " bytes.\n", filesize);
 
   /* check for empty file */
   if (filesize == 0) {
     printf("[-] The source file is empty.\n\n");
     fclose(pcap);
     fclose(pcap_fix);
+
     if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+    free(filename_fix);
+
     return(-4);
   }
 
@@ -306,7 +309,10 @@ int main(int argc, char *argv[]) {
     printf("[-] Cannot read file header (file too small?).\n\n");
     fclose(pcap);
     fclose(pcap_fix);
+
     if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+    free(filename_fix);
+
     return(-5);
   }
   fseeko(pcap, 0, SEEK_SET);
@@ -324,6 +330,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to no changes failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(-6);
 
@@ -338,6 +345,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to no changes failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(-6);
 
@@ -351,6 +359,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to no changes failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(-6);
 
@@ -363,6 +372,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to no changes failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(-6);
 
@@ -416,9 +426,8 @@ int main(int argc, char *argv[]) {
       fclose(pcap_fix);
 
       /* delete output file due to no changes failure */
-      if ((strcmp(filename, filename_fix) != 0) && (0 == keep_outfile)) {
-        remove(filename_fix);
-      }
+      if ((strcmp(filename, filename_fix) != 0) && (0 == keep_outfile)) remove(filename_fix);
+      free(filename_fix);
 
       return(0);
 
@@ -432,6 +441,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       /* deep scan / soft mode dependent output */
       if (deep_scan == 0 || soft_mode == 0) printf("If you are SURE that there are pcap packets inside, try with deep scan (-d) (pcap only!) and/or soft mode (-s) to find them!\n\n");
@@ -453,6 +463,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(res-10);
 
@@ -470,6 +481,7 @@ int main(int argc, char *argv[]) {
 
       /* delete output file due to failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      free(filename_fix);
 
       return(res-10);
   }
@@ -484,6 +496,7 @@ int main(int argc, char *argv[]) {
     fclose(pcap_fix);
     int success = truncate(filename_fix, finalpos);
     if (success != 0) printf("[-] Truncating result file failed!");
+    free(filename_fix);
 
     printf("[+] SUCCESS: %d Corruption(s) fixed!\n\n", res);
     return(0);
