@@ -1,6 +1,6 @@
 /*******************************************************************************
  *
- * Copyright (c) 2012-2019 Robert Krause (ruport@f00l.de)
+ * Copyright (c) 2012-2021 Robert Krause (ruport@f00l.de)
  *
  * This file is part of Pcapfix.
  *
@@ -115,13 +115,13 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
   // we use a buffer to cache 1mb of writing... this way writing is faster and
   // we can read and write the file at the same time
-  char *writebuffer;
-  uint64_t writepos = 0;
+  char *writebuffer, *tmpbuf;
+  off_t writepos = 0;
 
-  uint64_t bytes;                           /* written bytes/blocks counter */
-  uint64_t padding;                         /* calculation for padding bytes */
-  uint64_t pos;                             /* current block position in input file */
-  uint64_t filesize;                        /* size of input file */
+  off_t bytes;                              /* written bytes/blocks counter */
+  off_t padding;                            /* calculation for padding bytes */
+  off_t pos;                                /* current block position in input file */
+  off_t filesize;                           /* size of input file */
   unsigned int block_pos;                   /* current position inside -new_block- to write further data to */
   unsigned int check;                       /* variable to check end of blocks sizes */
   unsigned int count;                       /* option / record counter to create EOO/EOR if necessary */
@@ -130,7 +130,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
   unsigned int step;                        /* step counter for progress bar */
   unsigned int packets;
 
-  int64_t left;                             /* bytes left to proceed until current blocks end is reached */
+  off_t left;                               /* bytes left to proceed until current blocks end is reached */
   int fixes;                                /* corruptions counter */
   int res;                                  /* return values */
 
@@ -152,7 +152,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
   /* loop every block inside pcapng file until end of file is reached */
 
-  while (pos < filesize-sizeof(bh)) {
+  while ( pos < (off_t) (filesize-sizeof(bh))) {
 
     /* print out progress bar if in non-verbose mode */
     if ((verbose == 0) && (5*(float)pos/(float)filesize > step)) {
@@ -164,11 +164,14 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
     bytes = fread(&bh, sizeof(bh), 1, pcap);
     if (bytes != 1) return -3;
 
-    /* check for invalid block length / file cut off */
-    if (bh.total_length > filesize-pos) {
-      /* block size is larger than bytes in input file */
+    /* check for invalid block length / file cut off OR block size < 12 bytes */
+    if (bh.total_length > filesize-pos || bh.total_length < 12) {
+      /* block size is invalid (too small / larger than bytes in input file) */
 
-      if (verbose >= 1) printf("[-] Block Length (%u) exceeds file size (%" PRIu64 ").\n", bh.total_length, filesize);
+      if (verbose >= 1) {
+        if (bh.total_length > filesize-pos) printf("[-] Block Length (%" PRIu16 ") exceeds file size (%" FMT_OFF_T ").\n", bh.total_length, filesize);
+        else printf("[-] Block Length (%" PRIu16 ") is too small.\n", bh.total_length);
+      }
 
       /* search for next valid block */
       if (verbose >= 1) printf("[*] Trying to align next block...\n");
@@ -178,7 +181,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
       if (res == 0) {
         /* another valid block has been found in the file */
 
-        if (verbose >= 1) printf("[+] GOT Next Block at Position %" PRIu64 "\n", ftello(pcap));
+        if (verbose >= 1) printf("[+] GOT Next Block at Position %" FMT_OFF_T "\n", ftello(pcap));
 
         /* adjust total blocks length to match next block */
         bh.total_length = ftello(pcap)-pos;
@@ -193,7 +196,16 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
       }
 
-      if (verbose >= 1) printf("[*] Assuming this blocks size as %u bytes.\n", bh.total_length);
+      if (bh.total_length < 12) {
+        printf("[-] Block too small ==> SKIPPING.\n");
+
+        /* reset input file pointer to next block */
+        fseeko(pcap, pos+bh.total_length, SEEK_SET);
+
+        continue;
+      }
+
+      if (verbose >= 1) printf("[*] Assuming this blocks size as %" PRIu32 " bytes.\n", bh.total_length);
       else printf("[-] Invalid Block size => CORRECTED.\n");
 
       /* reset input file pointer behind block header */
@@ -210,15 +222,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
     new_block = malloc(bh.total_length);
 
     /* copy the current blocks header into repaired block */
-    memcpy(new_block, &bh, 8);
-    block_pos = 8;
+    memcpy(new_block, &bh, sizeof(bh));
+    block_pos = sizeof(bh);
 
     /* what is the type of block at current position ? */
     switch (bh.block_type) {
 
       /* Section Header Block */
       case TYPE_SHB:
-        if (verbose >= 1) printf("[*] FOUND: Section Header Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Section Header Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", pos, bh.total_length);
 
         /* read section header block into struct */
         bytes = fread(&shb, sizeof(shb), 1, pcap);
@@ -264,6 +276,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           shb.section_length = -1;
         }
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(shb)) {
+          bh.total_length = block_pos+sizeof(shb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy section header block into repaired block */
         memcpy(new_block+block_pos, &shb, sizeof(shb));
         block_pos += sizeof(shb);
@@ -280,29 +301,29 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           switch (oh.option_code) {
             /* End of Options */
             case 0x00:
-              if (verbose >= 2) printf("[+] OPTION: End of Options... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: End of Options... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Comment Option */
             case 0x01:
-              if (verbose >= 2) printf("[+] OPTION: Comment... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Comment... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Hardware Information */
             case 0x02:
-              if (verbose >= 2) printf("[+] OPTION: Hardware... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Hardware... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Operating System Information */
             case 0x03:
-              if (verbose >= 2) printf("[+] OPTION: Operating System... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Operating System... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* User Application Information */
             case 0x04:
-              if (verbose >= 2) printf("[+] OPTION: Userappl... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Userappl... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
           }
 
           /* Invalid Option? */
           if (oh.option_code > 0x04) {
-            printf("[-] Unknown option code: 0x%04x (%u bytes) ==> SKIPPING.\n", oh.option_code, oh.option_length);
+            printf("[-] Unknown option code: 0x%04x (%" PRIu16 " bytes) ==> SKIPPING.\n", oh.option_code, oh.option_length);
 
             /* increase corruptions counter */
             fixes++;
@@ -331,7 +352,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -352,6 +373,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -364,6 +394,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* write option data to repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -382,7 +421,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
         /* check oversize */
         if (sizeof(pb) > (unsigned)left) {
-          printf("[-] Packet #%u exceeds size of block header (%" PRIu64 " > %" PRId64 ") ==> SKIPPING.\n", packets, sizeof(pb), left);
+          printf("[-] Packet #%u exceeds size of block header (%zu > %" FMT_OFF_T ") ==> SKIPPING.\n", packets, sizeof(pb), left);
           /* set to "invalid block" */
           bh.block_type = -1;
           break;
@@ -399,7 +438,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND Packet #%u: Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Packet Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", packets, pos, bh.total_length);
 
         /* read packet block into struct */
         bytes = fread(&pb, sizeof(pb), 1, pcap);
@@ -409,7 +448,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         /* pre-check too large interface number (1024) */
         if (pb.interface_id > 1024) {
           /* interface id is unusal high --> this field is probably corrupted */
-          printf("[-] Probably corrupted Interface ID #%u (too high?) ==> CORRECTED.\n", pb.interface_id);
+          printf("[-] Probably corrupted Interface ID #%" PRIu32 " (too high?) ==> CORRECTED.\n", pb.interface_id);
           pb.interface_id = 1;
           fixes++;
         }
@@ -417,7 +456,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         /* check for the mandatory IDB that MUST identify every packets interface_id */
         while (pb.interface_id >= idb_num) {
           /* no IDB is identifying this packet, we need to create one - until the ID has been reached */
-          printf("[-] Missing IDB for Interface #%u ==> CREATING (#%u).\n", pb.interface_id, idb_num);
+          printf("[-] Missing IDB for Interface #%" PRIu32 " ==> CREATING (#%u).\n", pb.interface_id, idb_num);
           write_idb(pcap_fix, writebuffer, &writepos);
 
           /* increase counters */
@@ -425,13 +464,22 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(pb)) {
+          bh.total_length = block_pos+sizeof(pb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy packet block into repaired block */
         memcpy(new_block+block_pos, &pb, sizeof(pb));
         block_pos += sizeof(pb);
 
         /* check for oversized caplen */
         if (pb.caplen > (unsigned)left) {
-          printf("[-] Capture length (%u) exceeds block size (%" PRId64 ") ==> CORRECTED.\n", pb.caplen, left);
+          printf("[-] Capture length (%" PRIu32 ") exceeds block size (%" FMT_OFF_T ") ==> CORRECTED.\n", pb.caplen, left);
           pb.caplen = left;
         }
 
@@ -444,6 +492,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         bytes = fread(data, padding, 1, pcap);
         left -= padding;
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+padding) {
+          bh.total_length = block_pos+padding;
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy packet data into repaired block */
         memcpy(new_block+block_pos, data, padding);
         block_pos += padding;
@@ -464,25 +521,25 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           switch (oh.option_code) {
             /* End of Options */
             case 0x00:
-              if (verbose >= 2) printf("[+] OPTION: End of Options... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: End of Options... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Comment Option */
             case 0x01:
-              if (verbose >= 2) printf("[+] OPTION: Comment... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Comment... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Link Layer Flags */
             case 0x02:
-              if (verbose >= 2) printf("[+] OPTION: Link Layer Flags... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Link Layer Flags... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
             /* Packet Hash */
             case 0x03:
-              if (verbose >= 2) printf("[+] OPTION: Packet Hash... (%u bytes)\n", oh.option_length);
+              if (verbose >= 2) printf("[+] OPTION: Packet Hash... (%" PRIu16 " bytes)\n", oh.option_length);
               break;
           }
 
           /* Invalid Option? */
           if (oh.option_code > 0x03) {
-            printf("[-] Unknown option code: 0x%04x (%u bytes) ==> SKIPPING.\n", oh.option_code, oh.option_length);
+            printf("[-] Unknown option code: 0x%04" PRIx16 " (%" PRIu16 " bytes) ==> SKIPPING.\n", oh.option_code, oh.option_length);
 
             /* increase corruptions counter */
             fixes++;
@@ -511,7 +568,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -532,6 +589,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -544,6 +610,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option data to repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -570,7 +645,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND Packet #%u: Simple Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Simple Packet Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", packets, pos, bh.total_length);
 
         /* read simple packet block */
         bytes = fread(&spb, sizeof(spb), 1, pcap);
@@ -580,6 +655,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         /* output data */
         if (verbose >= 1) printf("[*] SPB original packet length: %u bytes\n", spb.len);
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(spb)) {
+          bh.total_length = block_pos+sizeof(spb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy simple packet block into repaired file */
         memcpy(new_block+block_pos, &spb, sizeof(spb));
         block_pos += sizeof(spb);
@@ -595,6 +679,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         bytes = fread(data, padding, 1, pcap);
         left -= padding;
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+padding) {
+          bh.total_length = block_pos+padding;
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy packet data into repaired block */
         memcpy(new_block+block_pos, data, padding);
         block_pos += padding;
@@ -625,6 +718,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         if (bytes != 1) return -3;
         left -= sizeof(idb);
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(idb)) {
+          bh.total_length = block_pos+sizeof(idb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy interface description block into repaired block */
         memcpy(new_block+block_pos, &idb, sizeof(idb));
         block_pos += sizeof(idb);
@@ -733,7 +835,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -754,6 +856,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -766,6 +877,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* write option data into repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -791,7 +911,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND: Name Resolution Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Name Resolution Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", pos, bh.total_length);
 
         /* process records */
         count = 0;
@@ -843,6 +963,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* record is valid */
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(nrb)) {
+            bh.total_length = block_pos+sizeof(nrb);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* write name resolution block into repaired block */
           memcpy(new_block+block_pos, &nrb, sizeof(nrb));
           block_pos += sizeof(nrb);
@@ -859,6 +988,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy record value into repaired buffer */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -934,7 +1072,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -955,6 +1093,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -967,6 +1114,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option value into repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -993,13 +1149,22 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND: Interface Statistics Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Interface Statistics Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", pos, bh.total_length);
 
         /* read interface statistics block */
         bytes = fread(&isb, sizeof(isb), 1, pcap);
         if (bytes != 1) return -3;
         left -= sizeof(isb);
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(isb)) {
+          bh.total_length = block_pos+sizeof(isb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy interface statistics block into repaired block */
         memcpy(new_block+block_pos, &isb, sizeof(isb));
         block_pos += sizeof(isb);
@@ -1084,7 +1249,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -1105,6 +1270,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -1117,6 +1291,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option value into repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -1143,7 +1326,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND Packet #%u: Enhanced Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Enhanced Packet Block at position %" FMT_OFF_T " (%" PRIu16 " bytes)\n", packets, pos, bh.total_length);
 
         /* read enhanced packet block */
         bytes = fread(&epb, sizeof(epb), 1, pcap);
@@ -1179,12 +1362,21 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
         /* check if packet capture size exceeds packet length */
         if (epb.caplen > left) {
-          printf("[-] Enhanced packet data exceeds total packet size (%u > %" PRIu64 ") ==> CORRECTED.\n", epb.caplen, left);
+          printf("[-] Enhanced packet data exceeds total packet size (%" PRIu32 " > %" FMT_OFF_T ") ==> CORRECTED.\n", epb.caplen, left);
           epb.caplen = left;
 
           fixes++;
         }
 
+        /* check if enough space is aligned in memory block */
+        if (bh.total_length < block_pos+sizeof(epb)) {
+          bh.total_length = block_pos+sizeof(epb);
+          printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+          new_block = realloc(new_block, bh.total_length);
+          memcpy(new_block, &bh, sizeof(bh));
+          fixes++;
+        }
+
         /* copy enhanced packet block into repaired buffer */
         memcpy(new_block+block_pos, &epb, sizeof(epb));
         block_pos += sizeof(epb);
@@ -1203,6 +1395,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           if (bytes != 1) return -3;
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy packet data into repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -1275,7 +1476,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* check oversize */
           if (padding > (unsigned)left) {
-            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            printf("[-] Option size (%" FMT_OFF_T ") exceeds remaining block space (%" FMT_OFF_T "). ==> SKIPPING OPTION.\n", padding, left);
             fixes++;
 
             /* because this block oversizes, there should not be any further option */
@@ -1296,6 +1497,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
             oh.option_length = 0x00;
           }
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+sizeof(oh)) {
+            bh.total_length = block_pos+sizeof(oh);
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -1308,6 +1518,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           bytes = fread(data, padding, 1, pcap);
           left -= padding;
 
+          /* check if enough space is aligned in memory block */
+          if (bh.total_length < block_pos+padding) {
+            bh.total_length = block_pos+padding;
+            printf("[-] Given block total length is too small ==> FIXED (%" PRIu32 ")\n", bh.total_length);
+            new_block = realloc(new_block, bh.total_length);
+            memcpy(new_block, &bh, sizeof(bh));
+            fixes++;
+          }
+
           /* copy option value into repaired block */
           memcpy(new_block+block_pos, data, padding);
           block_pos += padding;
@@ -1330,11 +1549,24 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
       /* increase corruption counter */
       fixes++;
 
+      free(new_block);
+
     } else {
       /* this block type is valid */
 
       /* write sizes of block header to correct positions */
       block_pos += sizeof(bh.total_length);
+
+      /* check size of block */
+      if (bh.total_length < block_pos) {
+        if (verbose >= 2) printf("[*] Increasing block buffer to %u\n", block_pos);
+        tmpbuf = malloc(block_pos);
+        memcpy(tmpbuf, new_block, bh.total_length);
+        free(new_block);
+        new_block = tmpbuf;
+        bh.total_length = block_pos;
+      }
+
       memcpy(new_block+4, &block_pos, sizeof(bh.total_length));
       memcpy(new_block+block_pos-4, &block_pos, sizeof(bh.total_length));
 
@@ -1351,13 +1583,21 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
       /* write repaired block into output file */
       if (verbose >= 2) printf("[*] Writing block to buffer (%u bytes).\n", block_pos);
 
-      // do we need to write the buffer to the file?
+      /* do we need to write the buffer to the file? */
       if (writepos + block_pos > 1024000) {
         bytes = fwrite(writebuffer, writepos, 1, pcap_fix);
         writepos = 0;
       }
 
-      // put new bytes into write buffer
+      /* check if writebuffer is large enough */
+      if (block_pos > 1024000) {
+        tmpbuf = malloc(block_pos);
+        memcpy(tmpbuf, writebuffer, 1024000);
+        free(writebuffer);
+        writebuffer = tmpbuf;
+      }
+
+      /* put new bytes into write buffer */
       memcpy(writebuffer+writepos, new_block, block_pos);
       writepos += block_pos;
 
@@ -1374,7 +1614,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
       if (verbose >= 2) printf("[+] End of Block reached... byte counter is correct!\n");
     } else {
       /* we did not read until end of block - maybe due to option skipping */
-      if (verbose >= 1) printf("[-] Did not hit the end of the block! (%" PRId64 " bytes left)\n", left);
+      if (verbose >= 1) printf("[-] Did not hit the end of the block! (%" FMT_OFF_T " bytes left)\n", left);
     }
 
     /* check for correct block end (block size) */
@@ -1402,8 +1642,8 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         res = find_valid_block(pcap, filesize);
 
         /* output information about overlapped/skipped bytes */
-        if (ftello(pcap) > (unsigned)bytes) printf("[-] Found %" PRId64 " bytes of unknown data ==> SKIPPING.\n", ftello(pcap)-bytes);
-        else printf("[-] Packet overlapps with %" PRId64 " bytes ==> CORRECTED.\n", bytes-ftello(pcap));
+        if (ftello(pcap) > (unsigned)bytes) printf("[-] Found %" FMT_OFF_T " bytes of unknown data ==> SKIPPING.\n", ftello(pcap)-bytes);
+        else printf("[-] Packet overlapps with %" FMT_OFF_T " bytes ==> CORRECTED.\n", bytes-ftello(pcap));
 
         /* increase corruption counter */
         fixes++;
@@ -1424,11 +1664,12 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
   }
 
-  // write remaining data into buffer
+  /* write remaining data into buffer */
   bytes = fwrite(writebuffer, writepos, 1, pcap_fix);
   writepos = 0;
 
   /* FILE HAS BEEN COMPLETELY CHECKED */
+  free(writebuffer);
 
   /* did we write any SHB blocks at all?
    * if not this seems to be no pcapng file! */
@@ -1450,8 +1691,8 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
  *          -1   error (reached EOF without finding a valid block)
  *
  */
-int find_valid_block(FILE *pcap, uint64_t filesize) {
-  uint64_t i;
+int find_valid_block(FILE *pcap, off_t filesize) {
+  off_t i;
   unsigned int bytes;
   unsigned int check;                       /* variable to check end of blocks sizes */
   struct block_header bh;                   /* block header */
@@ -1523,7 +1764,7 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
       if (check == bh.total_length) {
         /* also the second block size value is correct! */
 
-        if (verbose >= 1) printf("[+] FOUND: Block (Type: 0x%08x) at Position %" PRIu64 "\n", bh.block_type, i);
+        if (verbose >= 1) printf("[+] FOUND: Block (Type: 0x%08" PRIx32 ", Length: %u) at Position %" FMT_OFF_T "\n", bh.block_type, bh.total_length, i);
 
         /* set pointer to next block position */
         fseeko(pcap, i, SEEK_SET);
@@ -1548,12 +1789,12 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
  *          -1   error (cannot write to output file)
  *
  */
-int write_shb(FILE *pcap_fix, char* writebuffer, uint64_t* writepos) {
+int write_shb(FILE *pcap_fix, char* writebuffer, off_t* writepos) {
   struct block_header bh;           /* block header */
   struct section_header_block shb;  /* section header block */
   struct option_header oh;          /* options header */
 
-  uint64_t bytes;              /* written bytes/blocks counter */
+  off_t bytes;              /* written bytes/blocks counter */
   unsigned int size = 0;            /* size of whole block */
   unsigned int padding;             /* padding of data */
   unsigned char *data;              /* data buffer */
@@ -1647,12 +1888,12 @@ int write_shb(FILE *pcap_fix, char* writebuffer, uint64_t* writepos) {
  *          -1   error (cannot write to output file)
  *
  */
-int write_idb(FILE *pcap_fix, char* writebuffer, uint64_t* writepos) {
+int write_idb(FILE *pcap_fix, char* writebuffer, off_t* writepos) {
   struct block_header bh;                   /* block header */
   struct interface_description_block idb;   /* interface description block */
   struct option_header oh;                  /* options header */
 
-  uint64_t bytes;              /* written bytes/blocks counter */
+  off_t bytes;                      /* written bytes/blocks counter */
   unsigned int size = 0;            /* size of whole block */
   unsigned int padding;             /* padding of data */
   unsigned char *data;              /* data buffer */
